#!/usr/bin/env python
import argparse
import logging
from enum import Enum
# import redshift_connector
import snowflake.connector as snw
import json
import os


class Connectors(str, Enum):
    SNOWFLAKE = "SNOWFLAKE"
    REDSHIFT = "REDSHIFT"
    GCS = "GCS"
    BIGQUERY = "BIGQUERY"
    S3 = "S3"
    ADLS = "ADLS"
    JDBC = "JDBC"


def snowflake(connector_data):
    try:
        conn = None
        token = connector_data.pop("token", None)
        if token is not None:
            # use token
            conn = snw.connect(
                user=connector_data["user"],
                account=connector_data["url"]
                .replace(".snowflakecomputing.com", "")
                .replace("https://", ""),
                warehouse=connector_data["warehouse"],
                schema=connector_data["schema"],
                database=connector_data["database"],
                authenticator=("oauth"),
                token=token,
            )
        else:
            # use password
            conn = snw.connect(
                user=connector_data["user"],
                password=connector_data["password"],
                account=connector_data["url"]
                .replace(".snowflakecomputing.com", "")
                .replace("https://", ""),
                warehouse=connector_data["warehouse"],
                schema=connector_data["schema"],
                database=connector_data["database"],
            )

        if connector_data.get("table", None):
            cursor = conn.cursor()
            cursor.describe("SELECT * FROM {}".format(connector_data["table"]))

        return 0
    except snw.Error as e:
        raise SystemExit(e)
    finally:
        if conn is not None:
            conn.close()


def test_s3(connector_data):
    pass


def redshift(connector_data):
    pass
    # try:
    #     conn = redshift_connector.connect(
    #         host='',
    #         port='',
    #         database='',
    #         user='',
    #         password=''
    #     )
    #     # cursor = conn.cursor()
    #     # cursor.execute("select count(1) from users")
    #     # result: tuple = cursor.fetchall()
    #     # print(result)
    #     conn.close()
    #     return 0
    # except redshift_connector.InterfaceError as e:
    #     logging.error(e)
    #     raise SystemExit(e)


def test_gcs(connection_data):
    pass


def bigquery(connection_data):
    pass


def test_adls(connection_data):
    pass


def test_jdbc(connection_data):
    pass


def handle_validation(connection_data):
    connector_type = connection_data["storageConnectorType"]
    logging.info("Connector type {} ".format(connector_type))

    if connector_type == Connectors.SNOWFLAKE:
        status = snowflake(connection_data)
    elif connector_type == Connectors.REDSHIFT:
        status = redshift(connection_data)
    elif connector_type == Connectors.S3:
        status = test_s3(connection_data)
    elif connector_type == Connectors.GCS:
        status = test_gcs(connection_data)
    elif connector_type == Connectors.BIGQUERY:
        status = bigquery(connection_data)
    elif connector_type == Connectors.ADLS:
        status = test_adls(connection_data)
    elif connector_type == Connectors.JDBC:
        status = test_jdbc(connection_data)
    else:
        raise ValueError("Invalid Storage connector type")

    if status == 0:
        print("Success")
        exit(0)


def main():
    parser = argparse.ArgumentParser(
        description="Test connection for storage connector"
    )
    parser.add_argument(
        "input_json",
        type=str,
        metavar="Storage Connector JSON file path",
        help="Input path to connector JSON",
    )
    parser.add_argument(
        "-v", action="store_true", help="Verbose logging (default is only ERROR level)"
    )

    args = parser.parse_args()

    logging.basicConfig(
        level=(logging.INFO if args.v else logging.ERROR),
        format="[%(levelname)s] %(message)s",
        handlers=[
            logging.FileHandler(
                os.path.join(
                    "/home",
                    os.path.basename(args.input_json).split(".")[0] + ".log",
                )
            )
        ],
    )
    # Opening JSON file
    logging.info("Reading input JSON at {} ".format(args.input_json))
    with open(args.input_json) as json_file:
        data = json.load(json_file)
        handle_validation(data)


if __name__ == "__main__":
    main()
